########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_custr.4gs/midlevel.4gl  1.6  Delta: 5/16/90
########################################################################

globals "globals.4gl"

#######################################################################
# "ok" functions:
#  - the following "ok" functions can be placed here:
#    ok_add(), ok_update(), ok_delete(), ok_find(), ok_browse(), ok_next(),
#    ok_prev(), ok_tab(), ok_options(), ok_exit(), and ok_bang().
#  - their purpose is to allow/disallow that function from the ring menu.
#  - if their return value is true, then the function is allowed, and
#    if it is false, then the function is not allowed.
#  - following is an example of a smart delete function:
#
#    ##################################################################
#    function ok_delete()
#    # returning true or false based upon ok to delete
#    ##################################################################
#    #
#        if posted = "Y"
#        then
#            error " Cannot delete a posted record "
#            return false
#        end if
#        return true
#    end function
#    # ok_delete()
#
#######################################################################

##################################################################
function ok_add()
# returning true or false based upon ok to add
# always returns true.  sets new_cust flag to true
##################################################################
#
    let new_cust = true
    let dtl_update = false
    let header_first = true
    return true
end function
# ok_add

##################################################################
function ok_update()
# returning true or false based upon ok to update
# always returns true.  sets new_cust flag to false
# sets indicators to control screen sequence, depending on which
# screen the update key is pressed from
##################################################################
#
    let new_cust = false
    if curr_form = "2"
    then
        let dtl_update = true
        let header_first = false
    else
        let dtl_update = false
        let header_first = true
    end if
    return true
end function
# ok_update

##################################################################
function ok_delete()
# returning true or false based upon ok to delete
##################################################################
#
    if m_rcntrc.ar_balanced = "Y"
    then
        if p_rcustr.acct_bal is not null and
           p_rcustr.acct_bal != 0
        then
            call fg_err(3)  # Cannot delete a customer with a balance
            return false
        end if
        if p_rcustr.on_acct_amt is not null and
           p_rcustr.on_acct_amt != 0
        then
            call fg_err(4)  # Can't delete a customer with an on account balance
            return false
        end if
    end if
    delete from stropend where
            stropend.cust_code = m_rcustr.cust_code
    return true
end function
# ok_delete()

#######################################################################
function mlh_init()
# returning the where clause string
#######################################################################
#  - this functions only purpose is to return a string that will be used
#    for the where clause in the initial document grouping that takes
#    place when the program is loaded.
#  - examples:
#   return "1=1" (select all rows)
#   return "1=2" (select no rows)
#   return "posted = 0" (select only unposted documents)
#   return "rowid = 1" (select only the first row - for control files)
#
    return "1=2"
end function
# mlh_init()

#######################################################################
function mlh_validate()
#######################################################################
#   mid level data validation.  this function is called before any
#   disk writes.  if informix tries to write invalid data, part of the row
#   will be written, and part of it won't.  sql bomb's out.  the data
#   should have been checked in ll_input(), but us programmers can't
#   be perfect all of the time.  instead of data being compromised, the
#   end user doesn't get to write that screen.  it ticks them off, but
#   at least they don't have partial records on disk.
#
    whenever error continue
    call llh_m_prep()
    validate m_rcustr.* like strcustr.*
    whenever error call error_handler
end function
# mlh_validate()

#######################################################################
function mld_validate()
#######################################################################
#   like above, only validation must occur for each line
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt
        call lld_m_prep(n)
        validate m_rshipr.* like strshipr.*
    end for
    whenever error call error_handler
end function
# mld_validate

#######################################################################
function mlh_clear()
#######################################################################
#
    initialize p_rcustr.* to null
    initialize m_rcustr.* to null
end function
# mlh_clear

#######################################################################
function mld_clear()
#######################################################################
#
    define
        n       smallint            # working number
    initialize m_rshipr.* to null    # clear the m_record
    # clear all p_ records in the array
    for n = 1 to rec1_max
        initialize p_rshipr[n].* to null   # clear the p_record
    end for
    let rec1_cnt = 0               # reset the number of active elements
end function
# mld_mclear()

#######################################################################
function mlh_construct()
# returning criteria
#######################################################################
#
    define
        criteria    char(256)   # selection criteria holding bin

    if curr_form = "2"
    then
        let curr_form = "1"
        display form screen1
    end if

    # construct the search criteria from the screen
    construct criteria
        on
            strcustr.cust_code,
            strcustr.bus_name,
            strcustr.taxable,
            strcustr.contact,
            strcustr.phone,
            strcustr.address1,
            strcustr.address2,
            strcustr.city,
            strcustr.state,
            strcustr.zip,
            strcustr.country,
            strcustr.spec_shipping,
            strcustr.ar_type,
            strcustr.stmt_cycle,
            strcustr.fin_chg,
            strcustr.credit_limit,
            strcustr.ar_acct_dflt,
            strcustr.ar_department_dflt,
            strcustr.terms_code,
            strcustr.act_grp,
            strcustr.spec_billing,
            strcustr.stmt_date,
            strcustr.stmt_amount,
            strcustr.last_pay_date,
            strcustr.on_acct_amt,
            strcustr.acct_bal
        from
            s_rcustr.cust_code,
            s_rcustr.bus_name,
            s_rcustr.taxable,
            s_rcustr.contact,
            s_rcustr.phone,
            s_rcustr.address1,
            s_rcustr.address2,
            s_rcustr.city,
            s_rcustr.state,
            s_rcustr.zip,
            s_rcustr.country,
            s_rcustr.spec_shipping,
            s_rcustr.ar_type,
            s_rcustr.stmt_cycle,
            s_rcustr.fin_chg,
            s_rcustr.credit_limit,
            s_rcustr.ar_acct_dflt,
            s_rcustr.ar_department_dflt,
            s_rcustr.terms_code,
            s_rcustr.act_grp,
            s_rcustr.spec_billing,
            s_rcustr.stmt_date,
            s_rcustr.stmt_amount,
            s_rcustr.last_pay_date,
            s_rcustr.on_acct_amt,
            s_rcustr.acct_bal
    return criteria
end function
# mlh_construct

#######################################################################
function mlh_define_cur(criteria)
#######################################################################
#
    define
        criteria    char(256),  # selection criteria holding bin
        find_specs  char(512),  # select sql specification
        curs_count  integer     # number of elements in the cursor

    if criteria = "1=2" then let sqlca.sqlerrd[3] = 0 return end if

    # get the count into curs_count
    let find_specs =
        "select count(*) from strcustr where ",
        criteria clipped
    prepare get_count from find_specs
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into curs_count
    close count_cursor
    # define the cursor
    let find_specs =
        "select rowid, cust_code",
        " from strcustr where ",criteria clipped,
        " order by cust_code"
    prepare get_stmt from find_specs
    declare ref_cursor scroll cursor for get_stmt
    open ref_cursor
    let sqlca.sqlerrd[3] = curs_count
    return
end function
#mlh_define_cur()


#######################################################################
function mlh_fetch_cur(curno)
# returning rowid in sqlca.sqlerrd[6]
#######################################################################
#
    define
        curno       integer,    # absolute cursor number
        curs_rowid integer, p1 char(1)

    if curno = 0 then let sqlca.sqlerrd[6] = -1  return end if

    fetch absolute curno ref_cursor into
            curs_rowid, p1
    let sqlca.sqlerrd[6] = curs_rowid
    return
end function
# mlh_fetch_cur()

#######################################################################
function mlh_close_cur()
#######################################################################
#
    close ref_cursor
end function
# mlh_close_cur()

#######################################################################
function mlh_lock(ring_rowid)
#######################################################################
#   - this function opens a cursor for update selecting just the
#     record defined by rowid.
#   - because it fetches the record with an update cursor, the record
#     is locked.
#   - it's only meaning in life is to lock the record. not read it.
#
    define
        ring_rowid  integer,    # rowid of record to read
        getid       integer     # something to fetch into
    whenever error continue
    declare lock_cursor cursor for
        select rowid into getid from strcustr where rowid = ring_rowid
        for update
    open lock_cursor
    fetch lock_cursor
    whenever error call error_handler
    return
end function
# mlh_lock()

#######################################################################
function mlh_unlock()
#######################################################################
#
    close lock_cursor
end function
# mlh_unlock()

#######################################################################
function mld_scroll()
#######################################################################
#   - this function calls display array for the window.
#

    if rec1_cnt < 1
    then
        return
    end if

    if curr_form = "1"
    then
        display form screen2
        call display_name()
    end if

    call set_count(rec1_cnt)
    display array p_rshipr to s_rshipr.*
        on key (control-i)
            exit display
    end display

    if curr_form = "1"
    then
        display form screen1
        call llh_display()
    end if

end function
# mld_scroll

########################################################################
function mld_arr_count()
########################################################################
#   - this function sets the rec1_cnt variable instead of informix's
#     arr_count() function that can't be trusted.
#
    define
        n       smallint        # working number
    for n = rec1_max to 1 step -1
        if      p_rshipr[n].ship_to_code  is not null
            or  p_rshipr[n].bus_name  is not null
            or  p_rshipr[n].taxable  is not null
            or  p_rshipr[n].contact  is not null
            or  p_rshipr[n].phone  is not null
            or  p_rshipr[n].address1  is not null
            or  p_rshipr[n].address2  is not null
            or  p_rshipr[n].city  is not null
            or  p_rshipr[n].state  is not null
            or  p_rshipr[n].zip  is not null
            or  p_rshipr[n].country  is not null
            or  p_rshipr[n].spec_shipping  is not null
        then
          if n <= arr_count()
          then
            let rec1_cnt = n
            return
          else
            let rec1_cnt = arr_count()
            initialize p_rshipr[rec1_cnt+1].* to null   # clear the p_record
            return
          end if
        end if
    end for
    let rec1_cnt = 0
    return
end function
# mld_arr_count()
