########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_invce.4gs/detail.4gl  1.7  Delta: 9/4/90
########################################################################

globals "globals.4gl"

#######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
#######################################################################
#   - scrolling input array
#   - must call set_count before input array
#

    define
        return_flag smallint

    if curr_form = 1
    then
        call r_inputd() returning return_flag
    else
        call q_inputd() returning return_flag
    end if
    return return_flag
end function
# lld_input()

#######################################################################
function lld_chkinput()
#   returns true/false based upon good/bad data in screen
#######################################################################
#   - this function is called upon exiting the input statement.
#     the "after input" clause in the input statement can't call this
#     because a tab (control-i) needs to run this function also.
#
#   if you have any must-fill type fields on the screen, you can use
#   the function "mustfill()" that comes with infourgen.  it's syntax is:
#       mustfill(variable_name,"description of that variable")
#           returning true/false based upon no_data/data in that field.
#   it's use in this context is:
#       if mustfill(p_gchrt.acct_no,"Account Number") then return false end if
#
    define
        counter smallint

    if curr_form = 1
    then
        for counter = 1 to rec1_max
            call set_acct_line(counter)
        end for
        call set_ar_line()
    else
        for counter = 1 to rec1_max
            let r_rinvcd[counter].tmp_amount = q_rinvcd[counter].tmp_amount
        end for
        let r_rinvce.ar_tmp = q_rinvce.ar_tmp
    end if
    return true
end function
# lld_chkinput()

#######################################################################
function lld_display()
#######################################################################
#   - display the first array elements onto the screen
#
    define
        n   smallint    # working number
    if curr_form = 1
    then
        for n = 1 to scr1_max
            display r_rinvcd[n].* to s_1invcd[n].*
        end for
    else
        for n = 1 to scr2_max
            display q_rinvcd[n].* to s_2invcd[n].*
        end for
    end if
    let r_rinvce.sub_total = p_rinvce.sub_total
    let q_rinvce.sub_total = p_rinvce.sub_total
    if curr_form = 1
    then
        display by name r_rinvce.sub_total
    else
        display by name q_rinvce.sub_total
    end if
end function
# lld_display()

#######################################################################
function lld_p_prep(n)
#######################################################################
#   - this function creates the p_* record array (element n)
#     from the m_* record
#
    define
        n   smallint,               # array element number
        acct_desc char(20),
        incr_with char(2)

    let p_rinvcd[n].acct_no = m_rinvcd.acct_no
    let p_rinvcd[n].department = m_rinvcd.department
    if m_rinvcd.amount is null
    then
        let m_rinvcd.amount = 0
    end if
    let p_rinvcd[n].amount = m_rinvcd.amount
    let p_rinvcd[n].debit_credit = m_rinvcd.debit_credit
    let p_rinvcd[n].description = m_rinvcd.description
    call credit_amount(p_rinvcd[n].debit_credit, p_rinvcd[n].amount)
        returning p_rinvcd[n].tmp_amount
    if p_rinvce.sub_total is null then let p_rinvce.sub_total = 0 end if
    let p_rinvce.sub_total = p_rinvce.sub_total + p_rinvcd[n].tmp_amount
    let p_rinvcd[n].item_no = m_rinvcd.item_no
    let p_rinvcd[n].quantity = m_rinvcd.quantity
    let p_rinvcd[n].pack = m_rinvcd.pack
    let p_rinvcd[n].price = m_rinvcd.price
    call lld_p_q(n)
    if q_rinvcd[n].description is null
    then
        call al_chrtn(p_rinvcd[n].acct_no) returning acct_desc, incr_with
        let q_rinvcd[n].description = acct_desc
    end if

end function
# lld_p_prep()

#######################################################################
function lld_m_prep(n)
#######################################################################
#   - this function creates the m_* record from the
#     p_* record array (element n) and from the elements of the
#     header record that ties the header to the detail lines.
#
    define
        n   smallint                # array element number

    call lld_q_p(n)

    # items from the header...
    let m_rinvcd.doc_no = m_rinvce.doc_no

    # items from the p_ record array...
    let m_rinvcd.acct_no = p_rinvcd[n].acct_no
    let m_rinvcd.department = p_rinvcd[n].department
    let m_rinvcd.amount = p_rinvcd[n].amount
    let m_rinvcd.debit_credit = p_rinvcd[n].debit_credit
    let m_rinvcd.description = p_rinvcd[n].description
    let m_rinvcd.item_no = p_rinvcd[n].item_no
    let m_rinvcd.quantity = p_rinvcd[n].quantity
    let m_rinvcd.pack = p_rinvcd[n].pack
    let m_rinvcd.price = p_rinvcd[n].price
    let m_rinvcd.line_no = n

end function
# lld_m_prep()

#######################################################################
function lld_read()
#######################################################################
#   - this function reads in all of the array elements from disk
#   - it uses a cursor, but doesn't lock any rows.
#   - it reads into the m_* record, and calls p_prep for all array elements
#
    define
        n           smallint    # working number

    whenever error continue
    declare d_cursor cursor for  # all strinvcd with line number > 0
        select * from strinvcd where
            strinvcd.doc_no = m_rinvce.doc_no and strinvcd.line_no > 0
                order by doc_no, line_no
    open d_cursor
    let n = 0
    foreach d_cursor into m_rinvcd.*
        let n = n + 1
        if n > rec1_max
        then
            error " Error: Too many rows in strinvcd to read. "
            let rec1_cnt = 0
            call set_count(rec1_cnt)
            close d_cursor
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(n)                  # move data from m_* to p_[].*
    end foreach
    let rec1_cnt = n
    call set_count(rec1_cnt)
    close d_cursor
    return
end function
# lld_read()

#######################################################################
function lld_delete()
#######################################################################
#   - this function is for deleting all rows in the array
#
    delete from strinvcd where
            strinvcd.doc_no = m_rinvce.doc_no
end function
# lld_delete()

#######################################################################
function lld_add()
#######################################################################
#   - this function is for writing all rows in the array
#
    define
        n   smallint    # working number
    whenever error continue
    for n = 1 to rec1_cnt       # for all active array elements
        call lld_m_prep(n)
        insert into strinvcd values(m_rinvcd.*)
    end for
    call lld_m_misc_prep()   # create misc detail line
    insert into strinvcd values(m_rinvcd.*)
    call lld_m_frght_prep()  # create freight detail line
    insert into strinvcd values(m_rinvcd.*)
    call lld_m_tax_prep()    # create tax detail line
    insert into strinvcd values(m_rinvcd.*)
    call lld_m_ar_prep()    # create ar total detail line
    insert into strinvcd values(m_rinvcd.*)
    whenever error call error_handler
    return
end function
# lld_add()

#######################################################################
function lld_p_q(n)
#######################################################################
#   - this function creates the p_* record array (element n)
#     from the m_* record
#
    define
        n   smallint                # array element number

    let q_rinvcd[n].acct_no = p_rinvcd[n].acct_no
    let q_rinvcd[n].department = p_rinvcd[n].department
    let q_rinvcd[n].amount = p_rinvcd[n].amount
    let q_rinvcd[n].tmp_amount = p_rinvcd[n].tmp_amount
    let q_rinvcd[n].debit_credit = p_rinvcd[n].debit_credit
    let q_rinvcd[n].description = p_rinvcd[n].description
    let r_rinvcd[n].item_no = p_rinvcd[n].item_no
    let r_rinvcd[n].quantity = p_rinvcd[n].quantity
    let r_rinvcd[n].description = p_rinvcd[n].description
    let r_rinvcd[n].pack = p_rinvcd[n].pack
    let r_rinvcd[n].price = p_rinvcd[n].price
    let r_rinvcd[n].tmp_amount = p_rinvcd[n].tmp_amount

end function
# lld_p_q(n)

#######################################################################
function lld_q_p(n)
#######################################################################
#   - this function creates the p_* record array (element n)
#     from the m_* record
#
    define
        n   smallint                # array element number

    let p_rinvcd[n].acct_no = q_rinvcd[n].acct_no
    let p_rinvcd[n].department = q_rinvcd[n].department
    let p_rinvcd[n].amount = q_rinvcd[n].amount
    let p_rinvcd[n].debit_credit = q_rinvcd[n].debit_credit
    let p_rinvcd[n].description = r_rinvcd[n].description
    let p_rinvcd[n].item_no = r_rinvcd[n].item_no
    let p_rinvcd[n].quantity = r_rinvcd[n].quantity
    let p_rinvcd[n].pack = r_rinvcd[n].pack
    let p_rinvcd[n].price = r_rinvcd[n].price

end function
# lld_q_p()

#######################################################################
function lld_m_misc_prep()
#######################################################################
#   - this function creates the m_* record from the misc info part
#     of the header record
#
    # items from the header...
    let m_rinvcd.doc_no = m_rinvce.doc_no
    let m_rinvcd.acct_no = p_rinvce.misc_acct_no
    let m_rinvcd.department = p_rinvce.misc_department
    let m_rinvcd.amount = p_rinvce.misc_amount
    let m_rinvcd.debit_credit = p_rinvce.misc_debit_credit
    let m_rinvcd.description = null
    let m_rinvcd.item_no = null
    let m_rinvcd.quantity = null
    let m_rinvcd.pack = null
    let m_rinvcd.price = null
    let m_rinvcd.line_no = -1

end function
# lld_m_misc_prep()

#######################################################################
function lld_m_frght_prep()
#######################################################################
#   - this function creates the m_* record from the frght info part
#     of the header record
#
    # items from the header...
    let m_rinvcd.doc_no = m_rinvce.doc_no
    let m_rinvcd.acct_no = p_rinvce.frght_acct_no
    let m_rinvcd.department = p_rinvce.frght_department
    let m_rinvcd.amount = p_rinvce.frght_amount
    let m_rinvcd.debit_credit = p_rinvce.frght_debit_credit
    let m_rinvcd.description = null
    let m_rinvcd.item_no = null
    let m_rinvcd.quantity = null
    let m_rinvcd.pack = null
    let m_rinvcd.price = null
    let m_rinvcd.line_no = -2

end function
# lld_m_frght_prep()

#######################################################################
function lld_m_tax_prep()
#######################################################################
#   - this function creates the m_* record from the tax info part
#     of the header record
#
    # items from the header...
    let m_rinvcd.doc_no = m_rinvce.doc_no
    let m_rinvcd.acct_no = p_rinvce.tax_acct_no
    let m_rinvcd.department = p_rinvce.tax_department
    let m_rinvcd.amount = p_rinvce.tax_amount
    let m_rinvcd.debit_credit = p_rinvce.tax_debit_credit
    let m_rinvcd.description = null
    let m_rinvcd.item_no = null
    let m_rinvcd.quantity = null
    let m_rinvcd.pack = null
    let m_rinvcd.price = null
    let m_rinvcd.line_no = -3

end function
# lld_m_tax_prep()

#######################################################################
function lld_m_ar_prep()
#######################################################################
#   - this function creates the m_* record from the ar info part
#     of the header record
#
    # items from the header...
    let m_rinvcd.doc_no = m_rinvce.doc_no
    let m_rinvcd.acct_no = p_rinvce.ar_acct_no
    let m_rinvcd.department = p_rinvce.ar_department
    let m_rinvcd.amount = p_rinvce.ar_amount
    let m_rinvcd.debit_credit = p_rinvce.ar_debit_credit
    let m_rinvcd.description = null
    let m_rinvcd.item_no = null
    let m_rinvcd.quantity = null
    let m_rinvcd.pack = null
    let m_rinvcd.price = null
    let m_rinvcd.line_no = -4

end function
# lld_m_ar_prep()
