########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../data/ar/data_io.4gl  1.4  Delta: 9/20/90
########################################################################

globals "globals.4gl"

#####################################################################
function data_load()
#####################################################################
# This function writes the dbload script for each table in the
# tab[] array, then runs dbload using that script (for each table)
#
    define
        c     char(1),         # storage for an input character
        comma char(1),         # storage for the comma character
        run_exit integer,      # value returned from run command
        old_outlevel smallint, # old output level holding bin
        scratch char(255)      # scratch string storage

    display "Loading data into tables..."
    display ""
    for gc_tab = 1 to gn_tab

        # no load/unload on view type tables
        if tab[gc_tab].viw_start != 0
          then continue for
        end if

        # default the del_filter (for no deletes)
        let del_filter = "1=0"

        # call the before schema trigger for the table
        if trigger("before load") = false
          then continue for
        end if

        display "Table: ",tab[gc_tab].tabname clipped

        # if there is anything that should be deleted
        # from this table before loading, do it now.
        if del_filter != "1=0"
        then
            let g_sql_str = "delete from ", tab[gc_tab].tabname clipped,
              " where ", del_filter clipped
            # no need to display the delete command.
            # it makes people real nervous
            let old_outlevel = g_outlevel
            let g_outlevel = 3
            call sql_exec()
            let g_outlevel = old_outlevel  # reset output level
        end if

        # write the dbload script
        start report oneline to "dbload.cmd"
        let scratch = "file ", g_quote, tab[gc_tab].tabname clipped,
            ".unl", g_quote, " delimiter ",g_quote, "|", g_quote,
            (tab[gc_tab].col_end - tab[gc_tab].col_start + 1)
            using " <<<", ";"
        output to report oneline(0,scratch) 

        let scratch = "insert into ",tab[gc_tab].tabname clipped, " ("
        output to report oneline(0,scratch)

        let comma = " "
        for gc_col = tab[gc_tab].col_start to tab[gc_tab].col_end
            if gc_col = tab[gc_tab].col_end
              then let comma = " "
              else let comma = ","
            end if
            let scratch = col[gc_col].colname clipped, comma
            output to report oneline(0,scratch)
        end for
        
        output to report oneline(0,");")
        finish report oneline

        # load data into database...
        # set the exit value to the 'wc' of the error table.
        # (0 if no errors, nonzero if the error table contains anything)
        let scratch =
            "mv dbload.cmd ", g_unload_dir clipped, " >/dev/null 2>&1",
            "; cd ", g_unload_dir clipped,
            "; dbload -d ",g_database clipped,
            " -n 10000 -c dbload.cmd -l ",
            tab[gc_tab].tabname clipped, ".err > /dev/null 2>&1",
            "; exit `wc ",
            tab[gc_tab].tabname clipped, ".err`"
        run scratch returning run_exit

        # test for a good data load
        case run_exit
          when 0
            # good data load
            let scratch = "cd ", g_unload_dir clipped,
              "; rm -f dbload.cmd ",
              tab[gc_tab].tabname clipped, ".err"
            run scratch
          otherwise
            # an error was detected during error load.
            let scratch = "cd ", g_unload_dir clipped,
              "; echo ----- >> ${tmpdir=/tmp}/fg.errlog",
              "; cat dbload.cmd ", tab[gc_tab].tabname clipped,
              ".err >> $tmpdir/fg.errlog"
            run scratch
            display "An error has been detected during data load."
            display "It has been logged into $tmpdir/fg.errlog."
            prompt "Do you want to continue ? (Y/N) " for char c
            if upshift(c) = "N" then exit program(1) end if

        end case
        # call the after load trigger for the table
        if trigger("after load") = false then end if
    end for

    let g_sql_str = "update statistics"
    call sql_exec()

end function
# data_load()

#####################################################################
function data_unload()
#####################################################################
#  This function unloads the data into an unload file in the 
#  specified unload directory by the name of (filename).unl
#
    define
        num_rows integer,    # number of rows selected
        num_cols integer,    # number of columns in table selected
        scratch char(255),   # scratch string storage
        c_row smallint,      # current row
        c char(1),           # generic character
        n smallint,          # generic counter
        m smallint,          # generic counter
        comma char(1)        # storage for the comma character

    display "Unloading data from tables..."
    display ""
    for gc_tab = 1 to gn_tab

        # no load/unload on view type tables
        if tab[gc_tab].viw_start != 0
          then continue for
        end if

        # default the unl_filter (for unload all rows)
        let unl_filter = "1=1"
        # default the order by clause (for no order)
        let order_by = ""

        # call the before schema trigger for the table
        if trigger("before unload") = false
          then continue for
        end if

        display "Table: ",tab[gc_tab].tabname clipped

        let g_sql_str = "select"
        let comma = " "
        for gc_col = tab[gc_tab].col_start to tab[gc_tab].col_end
            let g_sql_str = g_sql_str clipped, comma,
              col[gc_col].colname
            let comma = ","
        end for
        let g_sql_str = g_sql_str clipped, " from ", tab[gc_tab].tabname

        # put in the filter if necessary
        if unl_filter != "1=1"
        then
            let g_sql_str = g_sql_str clipped,
              " where ", unl_filter clipped
        end if
        
        # put in the order by clause if necessary
        if order_by is not null
        then
            let g_sql_str = g_sql_str clipped,
              " order by ", order_by clipped
        end if

        call data_select() returning num_rows

        # output the rows
        let num_cols = tab[gc_tab].col_end - tab[gc_tab].col_start + 1
        let scratch = g_unload_dir clipped, "/",
          tab[gc_tab].tabname clipped, ".unl"
        start report oneline to scratch
        # for all rows selected
        for c_row = 1 to num_rows
            let scratch = " "
            let n = 1
            call data_fetch(c_row)
            # build the unload line
            for gc_col = 1 to num_cols
                # go through each character of the data
                # to make sure there are no "|" characters.
                # escape it if there is one found.
                for m = 1 to length(data[gc_col])
                    let c = data[gc_col][m,m]
                    if c = "|"
                    then
                        let scratch[n,n] = g_bkslsh
                        let n = n + 1
                    end if
                    let scratch[n,n] = c
                    let n = n + 1
                end for  # all characters in the column
                let scratch[n,n] = "|"  # insert the column seperator
                let n = n + 1
            end for      # all columns in the row
            output to report oneline(0,scratch)
        end for          # all rows selected
        finish report oneline
    end for              # all tables defined

    # now remove all of the last blank lines that the
    # informix report routine inadvertantly creates
    start report oneline to "unlfix.sh"
    output to report oneline(0,":")
    output to report oneline(0,"# unlfix.sh")
    output to report oneline(0,"# temporary shell script that is run from dbmerge")
    output to report oneline(0,"# to fix the last line of all unload files")
    output to report oneline(0,"")
    let scratch = "trap ", g_quote, "rm -f unlfix.sh", g_quote, " 2"
    output to report oneline(0,scratch)
    output to report oneline(0,"")
    output to report oneline(0,"for file in *.unl")
    output to report oneline(0,"do")
    let scratch = "    sed ", g_quote, "/^$/d", g_quote, " < $file > unlfix.tmp"
    output to report oneline(0,scratch)
    output to report oneline(0,"    mv unlfix.tmp $file")
    output to report oneline(0,"done")
    output to report oneline(0,"rm -f unlfix.sh")
    finish report oneline
    let scratch = "mv unlfix.sh ", g_unload_dir clipped,
                    "; cd ", g_unload_dir clipped,
                    "; chmod 777 ./unlfix.sh ; ./unlfix.sh"
    run scratch
end function
# data_unload()

#####################################################################
function data_select()
# returning the number of rows selected
#####################################################################
#  This function opens the cursor for the generic select statement
#  found in g_sql_str.  It is to be used with the data_fetch()
#  function to gather up to 100 columns of data from any select
#  statement.
#
    define
        curs_cnt integer    # count of the elements selected

    whenever error continue
    prepare run_cmd from g_sql_str
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        display "SQL ERROR"
        display g_sql_str clipped
        sleep 10
        exit program(1)
    end if
    declare cur scroll cursor for run_cmd

    # count the elements
    let curs_cnt = 0
    whenever error continue
    open cur
    while true
        fetch next cur into data[1]
        if sqlca.sqlcode = 0
            then let curs_cnt = curs_cnt + 1
            else exit while
        end if
    end while
    whenever error call error_handler

    return curs_cnt
end function
# data_select

#####################################################################
function data_fetch(n)
#####################################################################
#  This function fetches each element of the data declared in the
#  data_select() function, and places each column into a seperate
#  element of the data[] array.
#
    define
        n integer    # element to fetch

fetch absolute n cur into
        data[ 1], data[ 2], data[ 3], data[ 4], data[ 5],
        data[ 6], data[ 7], data[ 8], data[ 9], data[10],
        data[11], data[12], data[13], data[14], data[15],
        data[16], data[17], data[18], data[19], data[20],
        data[21], data[22], data[23], data[24], data[25],
        data[26], data[27], data[28], data[29], data[30],
        data[31], data[32], data[33], data[34], data[35],
        data[36], data[37], data[38], data[39], data[40],
        data[41], data[42], data[43], data[44], data[45],
        data[46], data[47], data[48], data[49], data[50],
        data[51], data[52], data[53], data[54], data[55],
        data[56], data[57], data[58], data[59], data[60],
        data[61], data[62], data[63], data[64], data[65],
        data[66], data[67], data[68], data[69], data[70],
        data[71], data[72], data[73], data[74], data[75],
        data[76], data[77], data[78], data[79], data[80],
        data[81], data[82], data[83], data[84], data[85],
        data[86], data[87], data[88], data[89], data[90],
        data[91], data[92], data[93], data[94], data[95],
        data[96], data[97], data[98], data[99], data[100]

end function
# data_fetch
