########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../data/ar/main.4gl  1.4  Delta: 9/20/90
########################################################################

globals "globals.4gl"

#####################################################################
main
#####################################################################
#
    defer interrupt
    whenever error call error_handler

    call init()
    call getargs()
    call define_data()

    if trigger("before init") = false
    then
        if sqlca.sqlcode < 1 then let sqlca.sqlcode = 1 end if
        exit program(sqlca.sqlcode)
    end if

    case
      when is_schema
        if open_db(true) = false
        then
            display "ERROR: Cannot create the specified database"
            exit program(1)
        end if
        call mod_schema()
      when is_load
        if open_db(true) = false
        then
            display "ERROR: Cannot create the specified database"
            exit program(1)
        end if
        call mod_schema()
        call data_load()
      when is_unload
        if open_db(false) = false
        then
            display "ERROR: Cannot open the specified database"
            exit program(1)
        end if
        call chk_schema()
        call data_unload()
      when is_index
        if open_db(false) = false
        then
            display "ERROR: Cannot open the specified database"
            exit program(1)
        end if
        call mod_index()
    end case

    exit program (0)
end main


#####################################################################
function init()
#####################################################################
#

    call logo()
    call b1_define()   # define the systables browser
    let g_quote = "\""
    let g_bkslsh = "\\"
    let g_outlevel = 4
    let g_err_action = "ask"
    let g_database = "standard"
    let g_unload_dir = "."
    let g_curr_brw = 1
    let gx_ren = 20   # maximum number of elements in the ren[] array
    let gx_drp = 20   # maximum number of elements in the drp[] array
    let gc_tab = 1

end function
# init()

#####################################################################
function getargs()
#####################################################################
#
    define
        n       smallint    # working number

    # set some defaults
    let is_schema = false
    let is_load = false
    let is_unload = false
    let is_index = false

    # parse away...
    for n = 1 to num_args()
        case
          when arg_val(n) = "-d"
            let g_database = arg_val(n+1)
          when arg_val(n) = "-s"
            let is_schema = true
            let is_unload = false
            let is_load = false
            let is_index = false
          when arg_val(n) = "-l"
            let is_schema = false
            let is_unload = false
            let is_load = true
            let is_index = false
            let g_unload_dir = arg_val(n+1)
          when arg_val(n) = "-u"
            let is_schema = false
            let is_unload = true
            let is_load = false
            let is_index = false
            let g_unload_dir = arg_val(n+1)
          when arg_val(n) = "-i"
            let is_schema = false
            let is_unload = false
            let is_load = false
            let is_index = true
        end case
    end for

    if   is_schema = false and is_load = false
     and is_unload = false and is_index = false
    then
        display
"syntax:"
        display
"  dbmerge.4ge [-d database]  -s | -i | -l directory | -u directory"
        display
""
        display
"  -d database specifies the database to use (default: standard)"
        display
"  -s tells dbmerge to update the schemas"
        display
"  -i tells dbmerge to (possibly drop and) build all defined indexes"
        display
"  -l tells dbmerge to load data from .unl files located in"
        display
"     the specified directory into database tables."
        display
"  -u tells dbmerge to unload data from database tables into"
        display
"     .unl files located in the specified directory."
        display
""
        display
"(note: the unload files are named (tablename).unl where tablename"
        display
"       is the name of the table that the unloaded data represents)"
        exit program(1)
    end if
end function
# getargs()

########################################################################
function logo()
########################################################################
#
    display "dbmerge"
    display "Copyright (c) 1990 FourGen Software Technologies, Inc."
    display "author: Loren West"
    display ""
end function
# logo()

########################################################################
function sql_exec()
########################################################################
#  This routine is called to execute an sql statement stored in the
#  global variable g_sql_str.
#
    define
        x smallint,             # temp. variable
        scratch char(2048),      # scratch string storage
        c char(1)               # yesno input variable

    # show it off...
    if g_outlevel > 3
    then
        display g_sql_str clipped
        display ""
    end if

    # prepare and execute the g_sql_str
    whenever error continue
    prepare exec_stmt from g_sql_str
    if sqlca.sqlcode = 0 then execute exec_stmt end if

    # handle the error
    if sqlca.sqlcode != 0
    then
        if trigger("before error") = false then end if
 
        # log the error
        let scratch = err_get(sqlca.sqlcode)
        let scratch = sqlca.sqlcode using "\n-<<<<: ", scratch clipped, "\n",
          "On the following SQL command:\n",
          g_sql_str clipped
        call err_log(scratch)
    end if

    # clear it out (we're done)
    let g_sql_str = ""

end function
# sql_exec()

########################################################################
function open_db(make_it)
# returning true/false  (false if it can't make it and needs to)
########################################################################
#  This function opens the database specified in g_database.
#  if make_it is true and it doesn't exist, then it will
#  create the database.  If make_it is false, and the db doesn't
#  exist, then the function will exit false.
#
    define
        make_it smallint        # make it if need be?

    whenever error continue
    database g_database
    whenever error call error_handler

    if sqlca.sqlcode != 0
    then
        # create the database if make_it = true
        if make_it then create database g_database end if

        # return false if make_it=false, or if the above
        # statement returned with a nonzero exit status.
        if sqlca.sqlcode != 0 then return false end if

        # this statement will only run if the database
        # was made above, and no errors occured.
        grant dba to public
    end if
    return true
end function
# open_db()

########################################################################
function askyn(mssg_string)
# returning true/false (based on "Yes" or "No" respectively)
########################################################################
#  This function opens a window with a menu for "YES" and "NO" selection
#
    define
        mssg_string char(72),        # message to be displayed
        true_false smallint          # menu item selected

    open window win_yn at 2,3 with 3 rows, 76 columns
      attribute(border)
    display mssg_string at 3,4
    menu ""
      command "Yes" "   Press [ENTER] for YES"
        let true_false = true
        exit menu
      command "No" "   Press [ENTER] for NO"
        let true_false = false
        exit menu
    end menu

    close window win_yn
    return true_false
end function
# askyn()

#####################################################################
function error_handler()
#####################################################################
#
    define
        scratch char(160)
    let scratch = "FATAL error:\n", err_get(sqlca.sqlcode) clipped
    call err_log(scratch)
    exit program(1)
end function
# error_handler()

#####################################################################
function err_log(scratch)
#####################################################################
#
    define
        scratch char(165),
        run_str char(200),
        c char(1)

    let run_str = "echo ------- dbmerge: `date` logname: `logname`",
      ">> ${tmpdir=/tmp}/fg.errlog"
    run run_str
    let run_str = "echo '", scratch clipped,
      "' >> ${tmpdir=/tmp}/fg.errlog"
    run run_str
    display "The following error has occured:"
    display scratch clipped
    display "It has been logged into $tmpdir/fg.errlog."
    case
      # g_err_action should contain one of the following:
      # continue, quit, or ask
      when g_err_action = "quit"
        exit program(1)
      when g_err_action = "ask"
        prompt "Do you want to continue ? (Y/N) " for char c
        if upshift(c) = "N" then exit program(1) end if
        if int_flag != 0 then exit program(1) end if
    end case
    display ""
    return
end function
# err_log()

#####################################################################
report oneline(tab_stop, line_str)
#####################################################################
# this report is designed for disk output one line at a time
# 
	define
		tab_stop smallint, 
		line_str	char(256)
	output
		left margin 0
		right margin 0
		top margin 0
		bottom margin 0
		page length 1
	format
		on every row
			print column tab_stop + 1, line_str clipped
end report
# oneline

#####################################################################
# Switching functions
# These "switches" have been hardcoded because only browses
# were needed, and only one of those were needed.
#####################################################################

function sw_define() call b1_define() end function
function sw_binit() call b1_init() end function
function sw_clear() call b1_clear() end function
function sw_close() call b1_close() end function
function sw_fetch() call b1_fetch() end function
function sw_lock() call b1_lock() end function
function sw_unlock() call b1_unlock() end function
