########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../data/ar/modify.4gl  1.4  Delta: 6/1/90
########################################################################

globals "globals.4gl"

#####################################################################
function chk_schema()
#####################################################################
#  This function checks the real schema based on the internal
#  definition of the schema & asks to continue if they are different
#
    display "Checking tables..."
    display ""
    for gc_tab = 1 to gn_tab
        display "Table: ",tab[gc_tab].tabname clipped
        if chk_table() = false
        then
            let g_sql_str = "'", tab[gc_tab].tabname clipped,
              "' is different from what I think it should be.",
              "  Continue ?"
            if askyn(g_sql_str) = false then exit program(1) end if
        end if
    end for
    display ""
end function
# chk_schema()

#####################################################################
function mod_schema()
#####################################################################
#  This function is called to modify all tables defined internally.
#  It calls mod_table for each table (if necessary).
#
    display "Updating schemas..."
    display ""
    for gc_tab = 1 to gn_tab
        # call the before schema trigger for the table
        if trigger("before schema") = false
          then continue for
        end if

        display "Table: ",tab[gc_tab].tabname clipped
        # Perform any rename or drop column statements
        call ren_drp(tab[gc_tab].tabname)

        # Load the new tables into the browser, and modify
        # as necessary.
        call mod_table()

        # call the after schema trigger for the table
        if trigger("after schema") = false then end if
    end for
    display ""
end function
# mod_schema()

#####################################################################
function mod_index()
#####################################################################
#  This function is called to drop and build all known indexes.
#
    display "Updating indexes..."
    display ""
    if trigger("before all indexes")
    then
        for gc_tab = 1 to gn_tab
            # call the before index trigger for the table
            if trigger("before index") = false
              then continue for
            end if
    
            display "Table: ",tab[gc_tab].tabname clipped
            # Load the new tables into the browser, and rebuild
            # their indexes as necessary.
            call new_index()
    
            # call the after index trigger for the table
            if trigger("after index") = false then end if
        end for
    end if
    display ""
end function
# mod_index()

#####################################################################
function ren_drp(tabname)
#####################################################################
#  This function renames columns based on the ren[] array for the
#  specified tabname.  It also drops columns based on the drp[]
#  array for the specified tabname.
#  It performs it's SQL work using the sql_exec() routine.
#
    define
        tabname char(18),  # table to work on.
        hld_action char(10)  # holding bin for current g_err_action

    let hld_action = g_err_action
    let g_err_action = "continue"

    # drop columns
    for gc_drp = 1 to gx_drp
        # check for more drops
        if drp[gc_drp].colname is null then exit for end if

        # build the sql string & execute it
        let g_sql_str = "alter table ", tabname clipped, " drop (",
          drp[gc_drp].colname clipped, ")"
        call sql_exec()

        # reset the drp[] array element
        initialize drp[gc_drp].* to null
    end for

    # rename columns
    for gc_ren = 1 to gx_ren
        # check for more renames
        if ren[gc_ren].old_colname is null then exit for end if

        # build the sql string & execute it
        let g_sql_str = "rename column ", tabname clipped, ".",
          ren[gc_ren].old_colname clipped, " to ",
          ren[gc_ren].new_colname clipped
        call sql_exec()

        # reset the ren[] array element
        initialize ren[gc_ren].* to null
    end for

    # reset previous error action
    let g_err_action = hld_action

end function
# ren_drp()

#####################################################################
function mod_table()
#####################################################################
#  This function modifies a table to make it consistent with the
#  internal definition of that table.  It never drops or renames
#  columns, but it will alter and add columns as necessary.
#  If the table is a view, then it will always drop and
#  re-create that view.
#
    define
        brw_hold smallint,               # current browseno holding bin
        comma char(1),                   # the comma character
        old_outlevel smallint,           # old output level holding bin
        is_found smallint,               # boolean: next column found?
        mod_type smallint,               # "safe" modification type
        mod_length smallint,             # "safe" modification length
        l smallint,                      # generic counter
        m smallint,                      # generic counter
        n smallint                       # generic counter

    # set the current browse number
    let brw_hold = g_curr_brw
    let g_curr_brw = num.b_syst

    # check to see if table exists (and load into browser)
    # If table doesn't exist, then build it.
    if load_systable(tab[gc_tab].tabname) = false
    then
        # if it's a view, create it
        if tab[gc_tab].viw_start != 0
        then
            call create_view()
            return  # no other modifications are necessary for views
        end if

        # not a view.  create the table
        let g_sql_str = "create table ",
          tab[gc_tab].tabname clipped, "("
        let comma = " "  # set comma for new statement
        for gc_col = tab[gc_tab].col_start to tab[gc_tab].col_end
            let g_sql_str = g_sql_str clipped, comma clipped, 
              col[gc_col].colname clipped, " ",
              col_type(col[gc_col].coltype, col[gc_col].collength)
            let comma = ","  # potentially more elements
        end for
        let g_sql_str = g_sql_str clipped, ")"
        call sql_exec()
        # load the new table into the browser
        if load_systable(tab[gc_tab].tabname) = true then end if
    end if

    # If this table is a view type, then drop the view
    # and re-create it to make sure that any changes are made.
    if tab[gc_tab].viw_start != 0
    then
        # set the output level to 0 (we don't need to see this output)
        let old_outlevel = g_outlevel
        let g_outlevel = 0

        let g_sql_str = "drop view ", tab[gc_tab].tabname clipped
        call sql_exec()
        call create_view()

        let g_outlevel = old_outlevel  # reset output level
        return  # no other modifications are necessary
    end if

    # check for the correct field definitions
    let g_curr_tab = num.t_syst.cols
    let g_sql_str = "alter table ", tab[gc_tab].tabname clipped
    let comma = " "
    for gc_col = tab[gc_tab].col_start to tab[gc_tab].col_end
        # check for no columns in the table
        if gc_col = 0 then exit for end if

        # look through the browse table for a colname match
        for n = 1 to brw[g_curr_brw].tab[g_curr_tab].arr_cnt
            if b_syst.t_cols[n].colname = col[gc_col].colname
            then
                # Found the column.  Check for correct type.
                # Translated logic:
                # IF...the internal column type doesn't match
                #      the column type from systables (and it's not
                #      just due to a 'not null' internal definition)
                # OR...it's a char, decimal, or money type column,
                #      and the length doesn't match
                # THEN modify the column
                # 
                if  (    b_syst.t_cols[n].coltype != col[gc_col].coltype
                     and b_syst.t_cols[n].coltype !=
                         col[gc_col].coltype - 256
                    )
                or  (
                       (  col[gc_col].coltype = 0
                       or col[gc_col].coltype = 5
                       or col[gc_col].coltype = 8
                       )
                     and
                        col[gc_col].collength !=
                          b_syst.t_cols[n].collength
                     )
                then
                    # calculate a "safe" modificaton length.
                    call safe_length(
                      b_syst.t_cols[n].coltype,   col[gc_col].coltype,
                      b_syst.t_cols[n].collength, col[gc_col].collength)
                      returning mod_length

                    # build the "modify" sql string.
                    # we can't modify a column that is "not null"
                    # so we must filter out the "not null" portion
                    if col[gc_col].coltype < 256
                      then let mod_type = col[gc_col].coltype
                      else let mod_type = col[gc_col].coltype - 256
                    end if
                    let g_sql_str = g_sql_str clipped, comma clipped,
                      " modify (", col[gc_col].colname clipped, " ",
                      col_type(mod_type, mod_length) clipped,
                      ")"
                    let comma = ","
                end if # column needs modifying

                # go on to next item in the col[] array
                exit for

            end if     # column matches

            # If we've gone through all items in the browser,
            # and still haven't found the column, then add it.
            if n = brw[g_curr_brw].tab[g_curr_tab].arr_cnt
            then
                # we can't add a column that is of "not null" type.
                # so we must change the type if it is of not null type.
                if col[gc_col].coltype < 256
                  then let mod_type = col[gc_col].coltype
                  else let mod_type = col[gc_col].coltype - 256
                end if

                let g_sql_str = g_sql_str clipped, comma clipped,
                  " add (", col[gc_col].colname clipped, " ",
                  col_type(mod_type, col[gc_col].collength)

                # determine where this column fits in the schema
                # (if it is "before" any known column)
                if gc_col < tab[gc_tab].col_end
                then
                    let is_found = false
                    # for all (internal) columns after the current one
                    for l = (gc_col + 1) to tab[gc_tab].col_end
                        if is_found then exit for end if
                        # try to find it in the browser
                        # (for all columns in the browser)
                        for m = 1 to
                          brw[g_curr_brw].tab[g_curr_tab].arr_cnt
                            if is_found then exit for end if
                            if b_syst.t_cols[m].colname = col[l].colname
                            then
                                # found a match.  insert the
                                # new column before this one.
                                let is_found = true
                                let g_sql_str = g_sql_str clipped,
                                  " before ", col[l].colname
                            end if
                        end for    # all columns in the browser
                    end for        # all known columns after this column
                end if             # any known columns after this column

                # finish off the add string
                let g_sql_str = g_sql_str clipped, ")"
                let comma = ","
            end if
        end for        # all columns in the browser
    end for            # all columns in the col[] array for this table

    # If any modifications are needed, then apply them now.
    if comma = "," then call sql_exec() end if

    return
end function
# mod_table()

#####################################################################
function create_view()
#####################################################################
#  This function will create a view
#
    define
        view_id integer,   # view table id
        x smallint         # scratch variable

    let g_sql_str = " "
    for gc_viw = tab[gc_tab].viw_start to tab[gc_tab].viw_end
        let g_sql_str = g_sql_str clipped, viw[gc_viw].v_text clipped
    end for
    let x = length(g_sql_str)
    if g_sql_str[x,x] = ";"
      then let g_sql_str[x,x] = "" end if
    call sql_exec()
end function
# create_view()

#####################################################################
function new_index()
#####################################################################
#  This function will drop and rebuild indexes if they are different.
#
    define
        brw_hold smallint,               # current browseno holding bin
        comma char(1),                   # the comma character
        rebuild smallint,                # boolean: rebuild the index?
        is_desc smallint,                # boolean: is this descending?
        n smallint                       # generic counter

    # set the current browse number
    let brw_hold = g_curr_brw
    let g_curr_brw = num.b_syst

    # check to see if table exists (and load into browser)
    # If table doesn't exist, then return.
    if load_systable(tab[gc_tab].tabname) = false then return end if

    # modify index definitions if necessary
    let g_curr_tab = num.t_syst.indx
    for gc_idx = tab[gc_tab].idx_start to tab[gc_tab].idx_end
        # check for no indexes in the table
        if gc_idx = 0 then exit for end if

        # if there aren't any indexes in the browser, then
        # rebuild it.  otherwise, checkout whats in the browser.
        if brw[g_curr_brw].tab[g_curr_tab].arr_cnt = 0
          then let rebuild = true
          else let rebuild = false
        end if

        # try to find a matching index
        for n = 1 to brw[g_curr_brw].tab[g_curr_tab].arr_cnt
            if b_syst.t_indx[n].idxname = idx[gc_idx].idxname
            then
                # found a name match.  check for other parts
                if  (b_syst.t_indx[n].idxtype = "U"
                    and idx[gc_idx].unique_idx != "Y")
                or  (b_syst.t_indx[n].idxtype != "U"
                    and idx[gc_idx].unique_idx != "N")
                  then let rebuild = true
                end if
                if  (b_syst.t_indx[n].clustered = "C"
                    and idx[gc_idx].clustered != "Y")
                or  (b_syst.t_indx[n].clustered != "C"
                    and idx[gc_idx].clustered != "N")
                  then let rebuild = true
                end if
                if b_syst.t_indx[n].part1 = idx[gc_idx].part[1]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part2 = idx[gc_idx].part[2]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part3 = idx[gc_idx].part[3]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part4 = idx[gc_idx].part[4]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part5 = idx[gc_idx].part[5]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part6 = idx[gc_idx].part[6]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part7 = idx[gc_idx].part[7]
                  then else let rebuild = true
                end if
                if b_syst.t_indx[n].part8 = idx[gc_idx].part[8]
                  then else let rebuild = true
                end if

                # drop the old index if it doesn't match
                if rebuild
                then
                    let g_sql_str = "drop index ", idx[gc_idx].idxname
                    call sql_exec()
                end if
                # we're done checking
                exit for
            end if  # found a match

            # have we run out of known indexes?
            if n = brw[g_curr_brw].tab[g_curr_tab].arr_cnt
              then let rebuild = true
            end if

        end for  # for all indexes in the browser

        # rebuild it if marked to do so.
        if rebuild
        then
            let comma = " "
            let g_sql_str = "create"
            if idx[gc_idx].unique_idx = "Y"
              then let g_sql_str = g_sql_str clipped, " unique"
            end if
            if idx[gc_idx].clustered = "Y"
              then let g_sql_str = g_sql_str clipped, " cluster"
            end if
            let g_sql_str = g_sql_str clipped, " index ",
              idx[gc_idx].idxname clipped, " on ",
              tab[gc_tab].tabname clipped, " ("

            let comma = " "
            for n = 1 to 8
                # are we done ?
                if idx[gc_idx].part[n] = 0 then exit for end if

                # check for descending
                if idx[gc_idx].part[n] < 0
                    then let is_desc = -1
                    else let is_desc = 1
                end if
    
                # set the current column pointer to the start of
                # the table + the column number (inversed if < 0)
                let gc_col = tab[gc_tab].col_start +
                  (idx[gc_idx].part[n] * is_desc) - 1
                let g_sql_str = g_sql_str clipped, comma clipped,
                  col[gc_col].colname

                # insert the desc verb if descending
                if is_desc = -1
                  then let g_sql_str = g_sql_str clipped, " desc"
                end if

                # set the comma for future concatenations
                let comma = ","
            end for

            # sql string is built.  terminate it, and exec it.
            let g_sql_str = g_sql_str clipped, ")"
            call sql_exec()
        end if   # needs rebuilding
    end for      # for all internal index definitions

    return
end function
# new_index()

#####################################################################
function chk_table()
# returning true/false based on table matching internal definition
#####################################################################
#  This function uses the tab[] array to determine if the table
#  in the database is the same as the internal definition in tab[].
#  It also loads the browser with all of the info about the table.
#
    define
        brw_hold smallint,               # current browseno holding bin
        n smallint                       # generic counter

    # set the tablename and current browse number
    let brw_hold = g_curr_brw
    let g_curr_brw = num.b_syst

    # check to see if table exists (and load into browser)
    if load_systable(tab[gc_tab].tabname) = false
    then
        let g_curr_brw = brw_hold
        return false
    end if

    # check for the correct field definitions
    let n = 1
    let g_curr_tab = num.t_syst.cols
    for gc_col = tab[gc_tab].col_start to tab[gc_tab].col_end
        # check for any columns in the table (probably a view)
        if gc_col = 0 then exit for end if

        # test for end of column array in browser
        if brw[g_curr_brw].tab[g_curr_tab].arr_cnt < n
          then return false
        end if
        # check for the names, types, and lengths
        if b_syst.t_cols[n].colname = col[gc_col].colname
          then else return false
        end if
        if b_syst.t_cols[n].coltype = col[gc_col].coltype
          then else return false
        end if
        if b_syst.t_cols[n].collength = col[gc_col].collength
          then else return false
        end if
        let n = n + 1
    end for

    # check for the correct index definitions
    let g_curr_tab = num.t_syst.indx
    for gc_idx = tab[gc_tab].idx_start to tab[gc_tab].idx_end
        # check for any indexes in the table
        if gc_idx = 0 then exit for end if

        # try to find a matching index
        for n = 1 to brw[g_curr_brw].tab[g_curr_tab].arr_cnt
            if b_syst.t_indx[n].idxname = idx[gc_idx].idxname
            then
                # found a name match.  check for other parts
                if  (b_syst.t_indx[n].idxtype = "U"
                    and idx[gc_idx].unique_idx != "Y")
                or  (b_syst.t_indx[n].idxtype != "U"
                    and idx[gc_idx].unique_idx != "N")
                  then return false
                end if
                if  (b_syst.t_indx[n].clustered = "C"
                    and idx[gc_idx].clustered != "Y")
                or  (b_syst.t_indx[n].clustered != "C"
                    and idx[gc_idx].clustered != "N")
                  then return false
                end if
                if b_syst.t_indx[n].part1 = idx[gc_idx].part[1]
                  then else return false
                end if
                if b_syst.t_indx[n].part2 = idx[gc_idx].part[2]
                  then else return false
                end if
                if b_syst.t_indx[n].part3 = idx[gc_idx].part[3]
                  then else return false
                end if
                if b_syst.t_indx[n].part4 = idx[gc_idx].part[4]
                  then else return false
                end if
                if b_syst.t_indx[n].part5 = idx[gc_idx].part[5]
                  then else return false
                end if
                if b_syst.t_indx[n].part6 = idx[gc_idx].part[6]
                  then else return false
                end if
                if b_syst.t_indx[n].part7 = idx[gc_idx].part[7]
                  then else return false
                end if
                if b_syst.t_indx[n].part8 = idx[gc_idx].part[8]
                  then else return false
                end if
                # we're done checking
                exit for
            end if

            # have we run out of known indexes?
            if n = brw[g_curr_brw].tab[g_curr_tab].arr_cnt
              then return false
            end if

        end for  # for all indexes in the browser
    end for      # for all internal index definitions

    # table checked out ok
    return true
end function
# chk_table()


#####################################################################
function load_systable(tabname)
# returning true/false based on table loading ok
#####################################################################
#  This function loads the browser with the specified tablename.
#
    define
        tabname char(18),  # table name
        brw_hold smallint                # current browseno holding bin

    # set the current browse number
    let brw_hold = g_curr_brw
    let g_curr_brw = num.b_syst

    let g_sel_construct = "tabname = ", g_quote,
      tabname clipped, g_quote
    call sw_binit()  # load the data into the browser
    call sw_close()  # close the browsers cursor

    if brw[g_curr_brw].curs_cnt = 1
    then
        let g_curr_brw = brw_hold
        return true
    else
        let g_curr_brw = brw_hold
        return false
    end if
end function
# load_table()

#####################################################################
function col_type(coltype, collength)
# returning a 30 character text string representing the column type
#####################################################################
#  This function is passed the (Informix) coded coltype and collength
#  and returns the column type as a string.
#
#  The coding scheme follows:
#     coltype:       collength:
#     0 = char(n)       n
#     1 = smallint      2
#     2 = integer       4
#     3 = float         sizeof(double)
#     4 = smallfloat    sizeof(float)
#     5 = decimal       (see note 2)
#     6 = serial        4
#     7 = date          4
#     8 = money         (see note 2)
#
#  NOTE 1:  Add 256 to the coltype value if the column
#           is of "NOT NULL" type also.
#  NOTE 2:  The collength argument is used when coltype is char, and
#           if the column is of type decimal or money.  In the decimal
#           or money situation, the length of the column is determined
#           using the following formula:
#
#             for m and n in decimal(m,n)
#               m = integer portion of: (collength/256)
#               n = collength - (m * 256)
#               ---- or, using "C": -----           
#               m = (collength >> 8) & 0xff
#               n = collength & 0xff
#
#           If the precision (n) is 255, then it is a "float" type.
#
#  Examples:
#    col_type(2,4)   returns "integer"
#    col_type(262,4) returns "serial not null"
#    col_type(264,4098) returns "money (16,2) not null"
#    col_type(264,2815) returns "money (10) not null"
#
    define
        coltype smallint,      # coded type
        collength smallint,  # coded length
        not_null smallint,   # boolean: is this a "not null" type?
        m smallint,          # storage for the total length
        n smallint,          # storage for the decimal precision
        scratch char(30)     # string to return

    let not_null = false

    if coltype >= 256
    then
        let not_null = true
        let coltype = coltype - 256
    end if

    # calculate the total and precision
    # if the type is decimal or money
    if coltype = 5 or coltype = 8
    then
        let m = collength / 256
        let n = collength - (256 * m)
    end if

    case coltype
      when 0 let scratch = "char(", collength using "<<<<<<", ")"
      when 1 let scratch = "smallint"
      when 2 let scratch = "integer"
      when 3 let scratch = "float"
      when 4 let scratch = "smallfloat"
      when 5
        if n = 255
          then let scratch = "decimal(", m using "<<<", ")"  # float
          else let scratch = "decimal(", m using "<<<",
                             ",", n using "<<<", ")"
        end if
      when 6 let scratch = "serial (",
        tab[gc_tab].serial_start using "<<<<<<", ")" # serial
      when 7 let scratch = "date"
      when 8
        if n = 255
          then let scratch = "money(", m using "<<<", ")"  # float
          else let scratch = "money(", m using "<<<",
                             ",", n using "<<<", ")"
        end if
    end case

    if not_null
    then
        let scratch = scratch clipped, " not null"
    end if

    # return the column type string
    return scratch
end function
# col_type()

#####################################################################
function safe_length(old_type, new_type, old_length, new_length)
# returning mod_length
#####################################################################
#  This function determines a safe column length for column
#  modification.  The rules are as follows:
#
#  - If it is a char type, then never decrease it.
#  - If it is a decimal or money type, then:
#    - never decrease the precision
#    - never decrease the length
#    - determine the best precision and length to change to.
#
    define
        old_type smallint,     # coded type (theirs)
        new_type smallint,     # coded type (ours)
        old_length smallint, # their length
        new_length smallint, # our length
        mod_length smallint, # compromise length
        old_m smallint,          # storage for the total length (old)
        old_n smallint,          # storage for the decimal precision
        new_m smallint,          # storage for the total length (new)
        new_n smallint,          # storage for the decimal precision
        mod_m smallint,          # storage for the total length (mod)
        mod_n smallint           # storage for the decimal precision

    case new_type
      when 0
        if old_length > new_length
          then return old_length
          else return new_length
        end if
      when 5 exit case
      when 8 exit case
      otherwise return new_length
    end case

    # For the program to get this far, the new column
    # type must be either decimal, or money
    # The following logic is for these 2 types only.

    # If the old column isn't the same type, then just
    # trust the new length to be correct enough.
    if old_type = 5 or old_type = 8
    then else
        return new_length
    end if

    # calculate old and new lengths and precisions.
    let old_m = old_length / 256
    let old_n = old_length - (256 * old_m)
    let new_m = new_length / 256
    let new_n = new_length - (256 * new_m)

    # if either old or new is a "float" type (ex: decimal(10))
    # then only increase the length if needed.
    # don't touch the precision.
    # If going to a float (new_n = 255), then no problem. But
    # If coming from a float type, some presision may be lost.
    if   new_n = 255
      or old_n = 255
    then
        let mod_n = new_n
        if old_m < new_m
          then let mod_m = new_m
          else let mod_m = old_m
        end if
        let mod_length = (mod_m * 256) + mod_n
        return mod_length
    end if

    # only increase precision
    if old_n < new_n
      then let mod_n = new_n
      else let mod_n = old_n
    end if

    # calculate the new length as the greater of
    # the two lengths on the "left" of the decimal
    # + the new length on the "right" of the decimal
    if (old_m - old_n) < (new_m - new_n)
      then let mod_m = (new_m - new_n) + mod_n
      else let mod_m = (old_m - old_n) + mod_n
    end if

    # calculate the new mod_length
    let mod_length = (mod_m * 256) + mod_n

    # return the safe modification length
    return mod_length
end function
# safe_length()
